package net.wanji.utc.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dto.CrossIdDTO;
import net.wanji.databus.dto.ExtendPhaseDTO;
import net.wanji.databus.dto.StepingPhaseDTO;
import net.wanji.databus.po.SaveToUtcPO;
import net.wanji.databus.vo.*;
import net.wanji.utc.service.control.ControlCommandStrategyService;
import net.wanji.utc.vo.SchemePhaseGreenTimeOptVO;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Objects;

import static net.wanji.utc.common.constant.Constants.*;

/**
 * 控制指令接口
 *
 * @author wuxiaokai
 * @date 2022/11/15 13:21:10
 */
@Slf4j
@RestController
@RequestMapping("/controlCommand")
@Api(value = "控制指令接口", description = "控制指令接口")
public class ControlCommandController {

    @Resource
    private ControlCommandStrategyService controlCommandStrategyService;

    /**
     * 信号机方案下发
     *
     * @param schemeSendVO 方案下发VO
     * @return
     */
    @AspectLog(description = "信号机方案下发-基础方案下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "信号机方案下发-基础方案下发", notes = "信号机方案下发-基础方案下发")
    @PostMapping("/schemeSend")
    public JsonViewObject schemeSend(@RequestBody @Validated SchemeSendVO schemeSendVO) throws Exception {
        return controlCommandStrategyService.schemeSendStrategy(schemeSendVO);
    }

    /**
     * 下发信号机计划
     *
     * @param planSendVO 计划下发VO
     * @return
     */
    @AspectLog(description = "计划下发-计划信息、时段信息", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "计划下发-计划信息、时段信息", notes = "计划下发-计划信息、时段信息")
    @PostMapping("/planSend")
    public JsonViewObject planSend(@RequestBody @Validated PlanSendVO planSendVO) throws Exception {
        return controlCommandStrategyService.planSendStrategy(planSendVO);
    }

    /**
     * 时间表下发
     *
     * @param scheduleSendVO 时间表下发VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "时间表下发-时间表信息", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "时间表下发-时间表信息", notes = "时间表下发-时间表信息")
    @PostMapping("/scheduleSend")
    public JsonViewObject scheduleSend(@RequestBody @Validated ScheduleSendVO scheduleSendVO) throws Exception {
        return controlCommandStrategyService.scheduleSendStrategy(scheduleSendVO);
    }

    /**
     * 相位配时下发-（下发相位参数）
     *
     * @return
     * @throws InterruptedException 中断异常
     */
    @AspectLog(description = "相位配时下发-（下发相位参数）", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "相位配时下发-（下发相位参数）", notes = "相位配时下发-（下发相位参数）")
    @PostMapping("/phaseTimingSend")
    public JsonViewObject phaseTimingSend(@RequestBody @Validated PhaseTimingSendVO phaseTimingSendVO) throws Exception {
        return controlCommandStrategyService.phaseTimingSendStrategy(phaseTimingSendVO);
    }

    /**
     * 全红控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "全红控制-路口全红控制/恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "全红控制-路口全红控制/恢复", notes = "全红控制-路口全红控制/恢复")
    @PostMapping("/allRedControl")
    public JsonViewObject allRedControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return controlCommandStrategyService.setSignalControlStrategy(commandVO.getCrossCode(), commandVO.getCommand(), ALL_RED_CONTROL);
    }

    /**
     * 黄闪控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "黄闪控制-路口黄闪控制/恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "黄闪控制-路口黄闪控制/恢复", notes = "黄闪控制-路口黄闪控制/恢复")
    @PostMapping("/yellowLightControl")
    public JsonViewObject yellowLightControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return controlCommandStrategyService.setSignalControlStrategy(commandVO.getCrossCode(), commandVO.getCommand(), YELLOW_LIGHT_CONTROL);
    }

    /**
     * 关灯控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "关灯控制-路口关灯控制/开灯", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "关灯控制-路口关灯控制/开灯", notes = "关灯控制-路口关灯控制/开灯")
    @PostMapping("/closeLightControl")
    public JsonViewObject closeLightControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return controlCommandStrategyService.setSignalControlStrategy(commandVO.getCrossCode(), commandVO.getCommand(), CLOSE_LIGHT_CONTROL);
    }

    /**
     * 步进控制
     *
     * @param crossCode 信号id
     * @param command   命令
     * @param stepNum   一步num
     * @return
     */
    @AspectLog(description = "步进控制-步进控制/恢复", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "步进控制-步进控制/恢复", notes = "步进控制-步进控制/恢复")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "crossCode", value = "13位路口编号", paramType = "query", required = true, dataType = "String"),
            @ApiImplicitParam(name = "command", value = "1 开始步进 0 取消步进", paramType = "query", required = true, dataType = "int"),
            @ApiImplicitParam(name = "stepNum", value = "0 顺序步进 n 跳过n个相位", paramType = "query", required = true, dataType = "int"),
    })
    @PostMapping("/stepControl")
    public JsonViewObject stepControl(@RequestParam String crossCode,
                                      @RequestParam Integer command,
                                      @RequestParam int stepNum) throws Exception {
        if (ObjectUtils.isEmpty(crossCode) || ObjectUtils.isEmpty(command) || ObjectUtils.isEmpty(stepNum)){
            return JsonViewObject.newInstance().fail("crossCode、command、stepNum 参数不能为空");
        }
        return controlCommandStrategyService.stepControlStrategy(crossCode, command, stepNum);
    }

    /**
     * 恢复时间表
     *
     * @param crossIdDTO
     * @return
     */
    @AspectLog(description = "恢复时间表-恢复路口时间表执行", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "恢复时间表-恢复路口时间表执行", notes = "恢复时间表-恢复路口时间表执行")
    @PostMapping("/recoverSchedule")
    public JsonViewObject recoverSchedule(@Validated @RequestBody CrossIdDTO crossIdDTO) throws Exception {
        return controlCommandStrategyService.recoverScheduleStrategy(crossIdDTO.getCrossId());
    }

    /**
     * 锁定控制
     *
     * @param commandVO 控制指令VO
     * @return
     * @throws Exception 异常
     */
    @AspectLog(description = "锁定控制-相位锁定/解锁", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "锁定控制-相位锁定/解锁", notes = "锁定控制-相位锁定/解锁")
    @PostMapping("/lockControl")
    public JsonViewObject lockControl(@RequestBody @Validated ControlCommandVO commandVO) throws Exception {
        return controlCommandStrategyService.lockControlStrategy(commandVO);
    }

    /**
     * 相位差下发
     */
    @AspectLog(description = "相位差下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "相位差下发", notes = "相位差下发")
    @PostMapping("/phaseDiffSend")
    public JsonViewObject phaseDiffSend() {
        return controlCommandStrategyService.phaseDiffSendStrategy();
    }

    @AspectLog(description = "删除信号机配置参数", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "删除信号机配置参数", notes = "删除信号机配置参数")
    @PostMapping("/delBaseConfig")
    public JsonViewObject delBaseConfig(@RequestBody @Validated DelBaseConfigVO delBaseConfigVO) throws Exception {
        return controlCommandStrategyService.delBaseConfig(delBaseConfigVO);
    }

    @AspectLog(description = "信号机校时", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "信号机校时", notes = "信号机校时")
    @PostMapping("/signalTiming")
    public JsonViewObject signalTiming(@RequestParam String crossCode,  Date date) throws Exception {
        if (Objects.isNull(date)) {
            date = new Date();
        }
        return controlCommandStrategyService.timing(crossCode, date);
    }

    @AspectLog(description = "临时方案", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "临时方案", notes = "临时方案")
    @PostMapping("/tempScheme")
    public JsonViewObject tempScheme(@RequestBody @Validated SchemeOptSendVO schemeOptSendVO) throws Exception {
        return controlCommandStrategyService.tempScheme(schemeOptSendVO);
    }

    /**
     * 临时方案下发
     */
    @AspectLog(description = "临时方案下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "临时方案下发", notes = "临时方案下发")
    @PostMapping("/tempSchemeDetail")
    public JsonViewObject tempSchemeDetail(@RequestBody @Validated TempSchemeSendVO tempSchemeSendVO)  throws Exception{
        return controlCommandStrategyService.tempSchemeDetail(tempSchemeSendVO);
    }

    @AspectLog(description = "下发数据到UTC", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "下发数据到UTC", notes = "下发数据到UTC")
    @PostMapping("/saveToUtc")
    public JsonViewObject saveToUtc(@RequestBody @Validated SaveToUtcPO saveToUtcPO) throws Exception {
        return controlCommandStrategyService.saveToUtc(saveToUtcPO);
    }

    /**
     * 海信-延长相位
     */
    @AspectLog(description = "海信-延长相位", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "海信-延长相位", notes = "海信-延长相位")
    @PostMapping("/extendPhase")
    public JsonViewObject extendPhase(@RequestBody @Validated ExtendPhaseDTO extendPhase)  throws Exception{
        return controlCommandStrategyService.extendPhase(extendPhase);
    }
    /**
     * 海信-步进相位
     */
    @AspectLog(description = "海信-步进相位", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "海信-步进相位", notes = "海信-步进相位")
    @PostMapping("/stepingPhase")
    public JsonViewObject stepingPhase(@RequestBody @Validated StepingPhaseDTO stepingPhaseDTO)  throws Exception{
        //调用信控系统的步进功能，该controller为了保证对外提供的接口参数统一放在body里
//        return controlCommandStrategyService.stepControlStrategy(stepingPhaseDTO.getCrossId(), stepingPhaseDTO.getCommand(), stepingPhaseDTO.getStepCount());
//        return controlCommandStrategyService.stepingPhase(stepingPhaseDTO);
        return controlCommandStrategyService.stepControlStrategy(stepingPhaseDTO.getCrossId(),stepingPhaseDTO.getCommand(),stepingPhaseDTO.getStepCount());
    }

    @AspectLog(description = "方案相位绿灯时间优化下发", operationType = BaseEnum.OperationTypeEnum.UPDATE)
    @ApiOperation(value = "方案相位绿灯时间优化下发", notes = "方案相位绿灯时间优化下发")
    @PostMapping("/schemePhaseGreenTimeOpt")
    public JsonViewObject schemePhaseGreenTimeOpt(@RequestBody @Validated SchemePhaseGreenTimeOptVO vo) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        try {
            jsonViewObject = controlCommandStrategyService.schemePhaseGreenTimeOpt(vo);
        } catch (Exception e) {
            return JsonViewObject.newInstance().fail("方案相位绿灯时间优化下发失败");
        }
        return jsonViewObject;
    }

}
