package net.wanji.utc.task;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.Constants;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.databus.dao.entity.*;
import net.wanji.databus.dao.mapper.*;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.vo.LightsStatusVO;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.cache.CrossPhaseDirTurnCache;
import net.wanji.utc.cache.CrossSchemePhaseTimeCountCache;
import net.wanji.utc.dto.CrossSchemePhaseCountDownDTO;
import net.wanji.utc.dto.PhaseCountDownDTO;
import net.wanji.utc.po.hk.CrossPhaseDirTurnPojo;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.time.LocalTime;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * @author duanruiming
 * @description 从信号机方案模拟信号机灯态
 * @date 2023/12/01 10:28
 */
//@Component
@Slf4j
@SuppressWarnings("unused")
public class SignalStatus4StaticSchemeTask {

    public static final Map<String, LightsStatusVO> currentCrossLightStatusCache = new HashMap<>();
    public static final Map<String, LightsStatusVO> runningStateInfoCache = new ConcurrentHashMap<>();

    @Resource
    private CrossSchedulesMapper crossSchedulesMapper;
    @Resource
    private CrossSectionMapper crossSectionMapper;
    @Resource
    private CrossSchemeMapper crossSchemeMapper;
    @Resource
    private CrossPhaseMapper crossPhaseMapper;
    @Resource
    private CrossLightsMapper crossLightsMapper;

    //@Scheduled(fixedRate = 1000)
    public void signalStatusCountDown() throws Exception {
        try {
            if (!currentCrossLightStatusCache.isEmpty()) {
                for (Map.Entry<String, LightsStatusVO> entry : currentCrossLightStatusCache.entrySet()) {
                    String crossId = entry.getKey();
                    LightsStatusVO lightsStatusVO = entry.getValue();
                    long timeStamp = Long.parseLong(lightsStatusVO.getTimeStamp());
                    String schemeStartTime = lightsStatusVO.getSchemeStartTime();
                    String phaseId = lightsStatusVO.getPhaseId();
                    String schemeId = lightsStatusVO.getSchemeId();
                    CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeNo(crossId, Integer.valueOf(schemeId));
                    executeNextPeriod(crossId, lightsStatusVO, phaseId, crossSchemePO);
                    // 当前方案下一周期
                    setDirLampGroupMap(lightsStatusVO);
                    if (Objects.nonNull(lightsStatusVO.getDirLampGroupMap())) {
                        // 当前时间戳自动加1s
                        lightsStatusVO.setTimeStamp(String.valueOf(timeStamp + 1000));
                        // 红灯倒计时
                        //getRedCountDown(crossId, lightsStatusVO, schemeId);
                        getRedCountDownV2(crossId, lightsStatusVO, schemeId);
                        Date date = new Date();
                        String currentTimeStr = String.valueOf(date.getTime());
                        lightsStatusVO.setTimeStamp(currentTimeStr);
                        runningStateInfoCache.put(crossId, lightsStatusVO);
                    }
                }
            }
        } catch (Exception e) {
            log.error("定时任务计算路口相位灯态倒计时异常", e);
            throw new Exception(e);
        }

    }

    /**
     * 每隔1分钟获取路口方案号
     *
     * @throws Exception
     */
    @Scheduled(fixedRate = 60 * 1000)
    public void getStaticCrossRunningLightStatus() throws Exception {
        try {
            List<CrossInfoPO> crossInfoList = CrossInfoCache.crossInfoList;
            for (CrossInfoPO crossInfoPO : crossInfoList) {
                String crossId = crossInfoPO.getId();
                String code = crossInfoPO.getCode();
                LightsStatusVO lightsStatusVO = new LightsStatusVO();
                List<CrossSectionPO> crossSectionPOS = crossSectionMapper.selectByCrossId(crossId);
                List<CrossSchedulesPO> crossSchedulesPOS = crossSchedulesMapper.selectByCrossIds(Collections.singletonList(crossId));
                if (!CollectionUtils.isEmpty(crossSchedulesPOS)) {
                    Map<Integer, List<CrossSchedulesPO>> schedulesMap = crossSchedulesPOS.stream().collect(Collectors.groupingBy(CrossSchedulesPO::getPlanId));
                    // 判断当前执行的调度，获取日计划号
                    Date date = new Date();
                    LocalTime currentTime = LocalTime.parse(DateUtil.getTime());
                    Integer currentPlanId = getCurrentPlanId(schedulesMap, date);
                    Integer schemeId = getCurrentSchemeId(crossSectionPOS, currentTime, currentPlanId);
                    CrossSchemePO crossSchemePO = crossSchemeMapper.selectSchemePOById(schemeId);
                    if (Objects.nonNull(crossSchemePO)) {
                        String schemeNo = crossSchemePO.getSchemeNo();
                        if (StringUtils.isNotBlank(schemeNo)) {

                            if (isNotNextSchemeNo(crossSchemePO)) {
                                continue;
                            }
                            initLightsStatus(crossId, code, lightsStatusVO, date, crossSchemePO, schemeNo);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("获取当前路口运行方案号失败：", e);
            throw new Exception(e);
        }
    }

    /**
     * 初始化灯态信息
     * @param crossId
     * @param code
     * @param lightsStatusVO
     * @param date
     * @param crossSchemePO
     * @param schemeNo
     */
    private static void initLightsStatus(String crossId, String code, LightsStatusVO lightsStatusVO, Date date, CrossSchemePO crossSchemePO, String schemeNo) {
        List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList = CrossSchemePhaseTimeCountCache.crossSchemePhaseCountDownList;
        for (CrossSchemePhaseCountDownDTO crossSchemePhaseCountDownDTO : crossSchemePhaseCountDownList) {
            if (StringUtils.equalsIgnoreCase(schemeNo, crossSchemePhaseCountDownDTO.getSchemeNo()) &&
                    StringUtils.equalsIgnoreCase(crossId, crossSchemePhaseCountDownDTO.getCrossId())) {
                lightsStatusVO.setCrossId(crossId);
                lightsStatusVO.setCyclePhaseCountDown(0);
                lightsStatusVO.setCycleCountDown(0);
                lightsStatusVO.setSchemeId(schemeNo);
                lightsStatusVO.setCycleLen(crossSchemePO.getCycle());
                lightsStatusVO.setPhaseId("0");
                String currentTimeStr = String.valueOf(date.getTime());
                lightsStatusVO.setTimeStamp(currentTimeStr);
                lightsStatusVO.setCode(code);
                lightsStatusVO.setRunMode("11");
                lightsStatusVO.setControlMode("11");
                lightsStatusVO.setManufacturerCode("STATIC");
                currentCrossLightStatusCache.put(crossId, lightsStatusVO);
            }
        }
    }

    private boolean isNotNextSchemeNo(CrossSchemePO crossSchemePO) {
        if (Objects.nonNull(crossSchemePO)) {
            String schemeNo = crossSchemePO.getSchemeNo();
            String crossId = crossSchemePO.getCrossId();
            LightsStatusVO lightsStatusVO = currentCrossLightStatusCache.get(crossId);
            if (Objects.nonNull(lightsStatusVO)) {
                String currentSchemeNo = lightsStatusVO.getSchemeId();
                return StringUtils.equalsIgnoreCase(schemeNo, currentSchemeNo);
            }
        }
        return false;
    }

    private static Integer getCurrentSchemeId(List<CrossSectionPO> crossSectionPOS, LocalTime currentTime, Integer currentPlanId) {
        if (!CollectionUtils.isEmpty(crossSectionPOS)) {
            List<CrossSectionPO> currentPlanIdSectionList = crossSectionPOS.stream().filter(po -> Objects.equals(currentPlanId, po.getPlanId())).collect(Collectors.toList());
            for (CrossSectionPO crossSectionPO : currentPlanIdSectionList) {
                Integer schemeId = crossSectionPO.getSchemeId();
                LocalTime startTime = LocalTime.parse(crossSectionPO.getStartTime());
                LocalTime endTime = LocalTime.parse(crossSectionPO.getEndTime());
                // 过滤非当前时段数据
                if (currentTime.isAfter(startTime) && currentTime.isBefore(endTime)) {
                    return schemeId;
                }
            }
        }
        return 0;
    }

    private static Integer getCurrentPlanId(Map<Integer, List<CrossSchedulesPO>> schedulesMap, Date currentDate) {
        for (Map.Entry<Integer, List<CrossSchedulesPO>> entry : schedulesMap.entrySet()) {
            Integer planId = entry.getKey();
            List<CrossSchedulesPO> schedulesPOS = entry.getValue();
            if (!CollectionUtils.isEmpty(schedulesPOS)) {
                for (CrossSchedulesPO schedulesPO : schedulesPOS) {
                    Integer isSpectialDay = schedulesPO.getWeek(); // 0 为特殊日期
                    Date specialDate = schedulesPO.getSpecialDate();
                    String currentDateStr = DateUtil.format(currentDate, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                    if (Objects.nonNull(specialDate)) {
                        String specialDate4DB = DateUtil.format(specialDate, Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND);
                        if (Objects.equals(0, isSpectialDay)) {
                            if (StringUtils.equalsIgnoreCase(currentDateStr.substring(0, 10), specialDate4DB.substring(0, 10))) {
                                return planId;
                            }
                        }
                    }
                    Integer week4DB = schedulesPO.getWeek();
                    int weekCurrent = DateUtil.getWeek(currentDate);
                    if (week4DB == weekCurrent || weekCurrent == 0) {
                        return planId;
                    }
                }
            }
        }
        return 0;
    }

    private static void getRedCountDownV2(String crossId, LightsStatusVO lightsStatusVO, String schemeId) throws Exception {
        Map<String, Object> phaseMap = new HashMap<>();
        List<CrossPhaseDirTurnPojo> crossPhaseDirTurnCache = CrossPhaseDirTurnCache.getCrossPhaseDirTurnCache(crossId);
        List<PhaseCountDownDTO> phaseCountDownList = getPhaseCountDownList(crossId, schemeId);
        if (!CollectionUtils.isEmpty(phaseCountDownList)) {
            // 获取当前运行相位号相序号
            int currentPhaseSortNo = 0;
            for (int i = 0; i < phaseCountDownList.size(); i++) {
                String phaseNo = phaseCountDownList.get(i).getPhaseNo();
                if (Objects.equals(lightsStatusVO.getPhaseId(), phaseNo)) {
                    currentPhaseSortNo = i;
                    break;
                }
            }
            Integer cycleLen = lightsStatusVO.getCycleLen();
            Integer cycleCountDown = lightsStatusVO.getCycleCountDown();
            int runtime = cycleLen - cycleCountDown;
            String currenPhaseId = lightsStatusVO.getPhaseId();
            for (int i = 0; i < phaseCountDownList.size(); i++) {
                Integer phaseTime = phaseCountDownList.get(i).getPhaseTime();
                Integer changePhaseTime = phaseCountDownList.get(i).getChangePhaseTime();
                String dirTurnPhaseNo = phaseCountDownList.get(i).getPhaseNo();
                int redCountDown = 0;
                if (i == currentPhaseSortNo) {
                    redCountDown = changePhaseTime - runtime;
                } else {
                    redCountDown = changePhaseTime - runtime - phaseTime;
                    if (redCountDown < 0) {
                        redCountDown = changePhaseTime - runtime - phaseTime + cycleLen;
                    }
                }
                // 遍历灯组缓存，获取相位号方向转向
                Map<Integer, Integer> turnCountDown = new HashMap<>();
                if (!CollectionUtils.isEmpty(crossPhaseDirTurnCache)) {
                    for (CrossPhaseDirTurnPojo turnPojo : crossPhaseDirTurnCache) {
                        if (StringUtils.equalsIgnoreCase(crossId, turnPojo.getCrossId())
                                && StringUtils.equalsIgnoreCase(schemeId, String.valueOf(turnPojo.getSchemeNo()))
                                && StringUtils.equalsIgnoreCase(dirTurnPhaseNo, String.valueOf(turnPojo.getPhaseNo()))) {

                            Integer phaseNoStr = turnPojo.getPhaseNo();

                            if (Objects.nonNull(turnPojo.getDirTurnMap())) {
                                for (Map.Entry<Integer, List<CrossLightsPO>> entry : turnPojo.getDirTurnMap().entrySet()) {
                                    Integer dir = entry.getKey();
                                    List<CrossLightsPO> crossLightsPOS = entry.getValue();
                                    if (!CollectionUtils.isEmpty(crossLightsPOS)) {
                                        for (CrossLightsPO crossLightsPO : crossLightsPOS) {
                                            Integer type = crossLightsPO.getType();
                                            Integer turn = crossLightsPO.getTurn();
                                            if (Objects.equals(type, 2)) { // 圆饼灯
                                                List<Integer> dirList = Arrays.asList(1, 2, 3);
                                                for (Integer commonTurn : dirList) {
                                                    turnCountDown.put(commonTurn, redCountDown);
                                                }
                                            } else {
                                                turnCountDown.put(turn, redCountDown);
                                            }
                                        }
                                    }
                                    setPhaseMap(phaseMap, dir, turnCountDown);
                                    //phaseMap.put(String.valueOf(dir), turnCountDown);
                                }
                            }
                        }
                    }
                }
            }

        }
        lightsStatusVO.setPhaseMap(phaseMap);
    }

    private static void getRedCountDown(String crossId, LightsStatusVO lightsStatusVO, String schemeId) throws Exception {
        Map<String, Object> phaseMap = new HashMap<>();
        List<CrossPhaseDirTurnPojo> crossPhaseDirTurnCache = CrossPhaseDirTurnCache.getCrossPhaseDirTurnCache(crossId);
        List<PhaseCountDownDTO> phaseCountDownList = getPhaseCountDownList(crossId, schemeId);
        if (!CollectionUtils.isEmpty(phaseCountDownList)) {
            for (PhaseCountDownDTO phaseCountDownDTO : phaseCountDownList) {
                Integer changePhaseTime = phaseCountDownDTO.getChangePhaseTime();
                Integer phaseTime = phaseCountDownDTO.getPhaseTime();
                Integer cycleLen = lightsStatusVO.getCycleLen();
                Integer cycleCountDown = lightsStatusVO.getCycleCountDown();
                String currenPhaseId = lightsStatusVO.getPhaseId();
                int runtime = cycleLen - cycleCountDown;
                int redCountDown = 0;
                if (!CollectionUtils.isEmpty(crossPhaseDirTurnCache)) {
                    for (CrossPhaseDirTurnPojo dirTurnPojo : crossPhaseDirTurnCache) {
                        if (StringUtils.equalsIgnoreCase(schemeId, String.valueOf(dirTurnPojo.getSchemeNo()))) {
                            if (!StringUtils.equalsIgnoreCase(currenPhaseId, String.valueOf(dirTurnPojo.getPhaseNo()))) {
                                Map<Integer, List<CrossLightsPO>> dirTurnMap = dirTurnPojo.getDirTurnMap();
                                for (Map.Entry<Integer, List<CrossLightsPO>> dirEntry : dirTurnMap.entrySet()) {
                                    Integer dir = dirEntry.getKey();
                                    Map<Integer, Integer> turnCountDown = new HashMap<>();
                                    List<CrossLightsPO> lightsPOS = dirEntry.getValue();
                                    if (!CollectionUtils.isEmpty(lightsPOS)) {
                                        // 忽略行人
                                        List<CrossLightsPO> person = lightsPOS.stream().filter(po -> po.getTurn() == 20).collect(Collectors.toList());
                                        if (person.size() == lightsPOS.size()) {
                                            continue;
                                        }
                                        Integer type = lightsPOS.get(0).getType();
                                        redCountDown = changePhaseTime - runtime - phaseTime;
                                        if (redCountDown <= 0) {
                                            redCountDown = cycleLen - runtime;
                                        }
                                        if (type == 2) { // 圆饼灯
                                            List<Integer> dirList = Arrays.asList(1, 2, 3);
                                            for (Integer commonTurn : dirList) {
                                                turnCountDown.put(commonTurn, redCountDown);
                                            }
                                        } else {
                                            Map<Integer, List<CrossLightsPO>> turnMap = lightsPOS.stream().collect(Collectors.groupingBy(CrossLightsPO::getTurn));
                                            for (Map.Entry<Integer, List<CrossLightsPO>> turnEntry : turnMap.entrySet()) {
                                                Integer turn = turnEntry.getKey();
                                                List<CrossLightsPO> value = turnEntry.getValue();
                                                for (CrossLightsPO lightsPO : value) {
                                                    if (lightsPO.getTurn() != 20) {
                                                        turnCountDown.put(turn, redCountDown);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                    setPhaseMap(phaseMap, dir, turnCountDown);
                                }
                            } else {
                                Integer cyclePhaseCountDown = lightsStatusVO.getCyclePhaseCountDown();
                                Map<Integer, List<CrossLightsPO>> dirTurnMap = dirTurnPojo.getDirTurnMap();
                                for (Map.Entry<Integer, List<CrossLightsPO>> dirEntry : dirTurnMap.entrySet()) {
                                    Integer dir = dirEntry.getKey();
                                    Map<Integer, Integer> turnCountDown = new HashMap<>();
                                    List<CrossLightsPO> lightsPOS = dirEntry.getValue();
                                    if (!CollectionUtils.isEmpty(lightsPOS)) {
                                        // 忽略行人
                                        List<CrossLightsPO> person = lightsPOS.stream().filter(po -> po.getTurn() == 20).collect(Collectors.toList());
                                        if (person.size() == lightsPOS.size()) {
                                            continue;
                                        }
                                        Integer type = lightsPOS.get(0).getType();
                                        redCountDown = cyclePhaseCountDown;
                                        if (type == 2) { // 圆饼灯
                                            List<Integer> dirList = Arrays.asList(1, 2, 3);
                                            for (Integer commonTurn : dirList) {
                                                turnCountDown.put(commonTurn, redCountDown);
                                            }
                                        } else {
                                            Map<Integer, List<CrossLightsPO>> turnMap = lightsPOS.stream().collect(Collectors.groupingBy(CrossLightsPO::getTurn));
                                            for (Map.Entry<Integer, List<CrossLightsPO>> turnEntry : turnMap.entrySet()) {
                                                Integer turn = turnEntry.getKey();
                                                List<CrossLightsPO> value = turnEntry.getValue();
                                                for (CrossLightsPO lightsPO : value) {
                                                    if (lightsPO.getTurn() != 20) {
                                                        turnCountDown.put(turn, redCountDown);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                    setPhaseMap(phaseMap, dir, turnCountDown);
                                }
                            }
                        }
                    }
                }
            }
        }
        lightsStatusVO.setPhaseMap(phaseMap);
    }

    /**
     * 统一方向转向倒计时
     * @param phaseMap
     * @param dir
     * @param turnCountDown
     */
    private static void setPhaseMap(Map<String, Object> phaseMap, Integer dir, Map<Integer, Integer> turnCountDown) {
        String dirStr = String.valueOf(dir);
        if (!phaseMap.isEmpty() && Objects.nonNull(phaseMap.get(dirStr))) {
            Map<Integer, Integer> turnCountDownCache = (Map<Integer, Integer>) phaseMap.get(dirStr);
            turnCountDown.putAll(turnCountDownCache);
            phaseMap.put(String.valueOf(dir), turnCountDown);
        } else {
            phaseMap.put(String.valueOf(dir), turnCountDown);
        }
    }

    /**
     * @param crossId
     * @param lightsStatusVO
     * @param crossSchemePO
     * @description 通过日计划缓存，切换下一个时段方案
     */
    private void executeNextSectionScheme(String crossId, LightsStatusVO lightsStatusVO, CrossSchemePO crossSchemePO) throws Exception {
        if (Objects.nonNull(crossSchemePO)) {
            List<CrossPhasePO> nextSchemePhaseList = crossPhaseMapper.selectByCrossIdAndPlanId(crossId, String.valueOf(crossSchemePO.getId()));
            Integer phaseTime = 0;
            String phaseId = null;

            if (!CollectionUtils.isEmpty(nextSchemePhaseList)) {
                for (CrossPhasePO crossPhasePO : nextSchemePhaseList) {
                    if (Objects.equals(1, crossPhasePO.getSort())) {
                        phaseTime = crossPhasePO.getPhaseTime();
                        phaseId = crossPhasePO.getPhaseNo();
                    }
                }
            }
            lightsStatusVO.setPhaseId(phaseId);
            lightsStatusVO.setCyclePhaseCountDown(phaseTime);
        }
    }

    /**
     * @param crossId
     * @param lightsStatusVO
     * @param phaseId
     * @param crossSchemePO
     * @description 切换下一方案周期
     */
    private void executeNextPeriod(String crossId, LightsStatusVO lightsStatusVO, String phaseId, CrossSchemePO crossSchemePO) throws Exception {
        if (lightsStatusVO.getCyclePhaseCountDown() <= 1 && Objects.nonNull(crossSchemePO)) {
            List<CrossPhasePO> crossPhasePOS = crossPhaseMapper.selectByCrossIdAndPlanId(crossId, String.valueOf(crossSchemePO.getId()));
            Integer nextPhaseTime = 0;
            String nextPhaseId = "0";
            if (!CollectionUtils.isEmpty(crossPhasePOS)) {
                for (int i = 0; i < crossPhasePOS.size(); i++) {
                    if (StringUtils.equalsIgnoreCase(phaseId, crossPhasePOS.get(i).getPhaseNo())) {
                        if (i == crossPhasePOS.size() - 1) {
                            nextPhaseTime = crossPhasePOS.get(0).getPhaseTime();
                            nextPhaseId = crossPhasePOS.get(0).getPhaseNo();
                        } else {
                            nextPhaseTime = crossPhasePOS.get(i + 1).getPhaseTime();
                            nextPhaseId = crossPhasePOS.get(i + 1).getPhaseNo();
                        }
                    }
                }
                if (StringUtils.equalsIgnoreCase(nextPhaseId, "0")) {
                    lightsStatusVO.setCycleCountDown(lightsStatusVO.getCycleLen());
                    lightsStatusVO.setPhaseId(crossPhasePOS.get(0).getPhaseNo());
                    lightsStatusVO.setCyclePhaseCountDown(crossPhasePOS.get(0).getPhaseTime());
                } else {
                    Integer cycleCountDown = lightsStatusVO.getCycleCountDown();
                    if (cycleCountDown != 1) {
                        lightsStatusVO.setCycleCountDown(--cycleCountDown);
                    } else {
                        lightsStatusVO.setCycleCountDown(lightsStatusVO.getCycleLen());
                    }
                    lightsStatusVO.setPhaseId(nextPhaseId);
                    lightsStatusVO.setCyclePhaseCountDown(nextPhaseTime);
                }
            }
        } else {
            lightsStatusVO.setCyclePhaseCountDown(lightsStatusVO.getCyclePhaseCountDown() - 1);
            Integer cycleCountDown = lightsStatusVO.getCycleCountDown();
            if (cycleCountDown >= 2) {
                lightsStatusVO.setCycleCountDown(--cycleCountDown);
            } else {
                lightsStatusVO.setCycleCountDown(lightsStatusVO.getCycleLen());
            }
        }
    }

    /**
     * @param lightsStatusVO
     * @description 通过当前周期倒计时方案相位灯态
     */
    public void setDirLampGroupMap(LightsStatusVO lightsStatusVO) throws Exception {
        String crossId = lightsStatusVO.getCrossId();
        String currentSchemeNo = lightsStatusVO.getSchemeId();
        String currentPhaseNo = lightsStatusVO.getPhaseId();
        Integer cyclePhaseCountDown = lightsStatusVO.getCyclePhaseCountDown();
        List<CrossPhaseDirTurnPojo> crossPhaseDirTurnCache = CrossPhaseDirTurnCache.getCrossPhaseDirTurnCache(crossId);
        if (!CollectionUtils.isEmpty(crossPhaseDirTurnCache)) {
            Map<String, Object> dirLampGroupMap = new HashMap<>();
            setPhaseLampColor(crossId, currentSchemeNo, currentPhaseNo, cyclePhaseCountDown, crossPhaseDirTurnCache, dirLampGroupMap);
            lightsStatusVO.setDirLampGroupMap(dirLampGroupMap);
        }
    }

    private void setPhaseLampColor(String crossId, String currentSchemeNo, String currentPhaseNo, Integer cyclePhaseCountDown, List<CrossPhaseDirTurnPojo> crossPhaseDirTurnCache, Map<String, Object> dirLampGroupMap) throws Exception {
        for (CrossPhaseDirTurnPojo phaseDirTurnPojo : crossPhaseDirTurnCache) {
            String schemeNo = String.valueOf(phaseDirTurnPojo.getSchemeNo());
            String phaseNo = String.valueOf(phaseDirTurnPojo.getPhaseNo());
            if (StringUtils.equalsIgnoreCase(schemeNo, currentSchemeNo)) {
                // 判断圆饼灯代表123，还是23
                List<CrossLightsPO> crossLightsPOS = crossLightsMapper.selectByCrossId(crossId);
                if (!StringUtils.equalsIgnoreCase(currentPhaseNo, String.valueOf(phaseNo))) {
                    // 相位红灯
                    Map<Integer, List<CrossLightsPO>> dirTurnMap = phaseDirTurnPojo.getDirTurnMap();
                    if (Objects.nonNull(dirTurnMap) && !dirTurnMap.isEmpty()) {
                        for (Map.Entry<Integer, List<CrossLightsPO>> entry : dirTurnMap.entrySet()) {
                            String dir = String.valueOf(entry.getKey());
                            Map<Integer, String> turnColor = null;
                            if (!dirLampGroupMap.containsKey(dir)) {
                                turnColor = new HashMap<>();
                            } else {
                                turnColor = (Map<Integer, String>) dirLampGroupMap.get(dir);
                            }
                            List<CrossLightsPO> value = entry.getValue();
                            int type = 1;
                            for (CrossLightsPO crossLightsPO : value) { //todo 需要处理，路口方向级别，灯组圆饼灯要区分，有没有左转，crossd dir级别，有几个转向
                                Integer turn = crossLightsPO.getTurn();
                                if (crossLightsPO.getType() != 3) { // 设置灯色时不处理行人
                                    type = crossLightsPO.getType();
                                }
                                // 搭接相位跳过绿灯变红灯
                                if (turnColor.containsKey(turn) && turnColor.get(turn).equals("green")) {
                                    continue;
                                }
                                turnColor.put(turn, "red");
                            }
                            List<CrossLightsPO> dirAllLights = crossLightsPOS.stream()
                                    .filter(po -> Objects.equals(po.getDir(), entry.getKey()) && !Objects.equals(po.getType(), 3))
                                    .collect(Collectors.toList());
                            if (type == 2) {
                                if (dirAllLights.size() == 1) {
                                    setAllTurnColor(turnColor, "red"); //红灯时，圆饼灯操作
                                } else {
                                    turnColor.put(2, "red");
                                }
                            }
                            turnColor.put(3, "green");
                            dirLampGroupMap.put(dir, turnColor);
                        }
                    }
                } else {
                    // 相位绿灯
                    PhaseCountDownDTO phaseCountDownDTO = getPhaseCountDownDTO(crossId, schemeNo, phaseNo);
                    if (Objects.isNull(phaseCountDownDTO)) {
                        continue;
                    }
                    Integer yellowTime = phaseCountDownDTO.getYellowTime();
                    Integer redTime = phaseCountDownDTO.getRedTime();
                    Map<Integer, List<CrossLightsPO>> dirTurnMap = phaseDirTurnPojo.getDirTurnMap();
                    if (Objects.nonNull(dirTurnMap) && !dirTurnMap.isEmpty()) {
                        setDirTurnColor(cyclePhaseCountDown, dirLampGroupMap, yellowTime, redTime, dirTurnMap, crossLightsPOS);
                    }
                }
            }
        }
    }

    private static void setDirTurnColor(Integer cyclePhaseCountDown, Map<String, Object> dirLampGroupMap,
                                        Integer yellowTime, Integer redTime, Map<Integer,
            List<CrossLightsPO>> dirTurnMap, List<CrossLightsPO> crossLightsPOS) throws Exception {
        for (Map.Entry<Integer, List<CrossLightsPO>> dirEntry : dirTurnMap.entrySet()) {
            String dir = String.valueOf(dirEntry.getKey());
            List<CrossLightsPO> lightsList = dirEntry.getValue();
            if (CollectionUtils.isEmpty(lightsList)) {
                continue;
            }
            Map<Integer, String> turnColor = null;
            if (!dirLampGroupMap.containsKey(dir)) {
                turnColor = new HashMap<>();
            } else {
                turnColor = (Map<Integer, String>) dirLampGroupMap.get(dir);
            }
            Integer type = 2; // 灯组类型：1箭头；2圆饼；3行人；
            String color = null;
            for (CrossLightsPO crossLightsPO : lightsList) {
                type = crossLightsPO.getType();
                Integer turn = crossLightsPO.getTurn();
                color = setTurnColor(cyclePhaseCountDown, yellowTime, redTime, turnColor, color, turn);
                turnColor.put(turn, color);
            }
            List<CrossLightsPO> dirAllLights = crossLightsPOS.stream()
                    .filter(po -> Objects.equals(po.getDir(), dirEntry.getKey()) && !Objects.equals(po.getType(), 3))
                    .collect(Collectors.toList());
            if (type == 2) { // 方向是圆饼灯，并且只有一个灯组
                if (dirAllLights.size() == 1) {
                    setAllTurnColor(turnColor, color);
                } else {
                    turnColor.put(2, "green");
                }
            }
            turnColor.put(3, "green"); // 右转常绿
            dirLampGroupMap.put(dir, turnColor);
        }
    }

    private static String setTurnColor(Integer cyclePhaseCountDown, Integer yellowTime, Integer redTime,
                                       Map<Integer, String> turnColor, String color, Integer turn) throws Exception {
        if (cyclePhaseCountDown > yellowTime + redTime) {
            color = "green";
            if (turn == 20) {
                turnColor.put(turn, "red");
            }
        }
        if (cyclePhaseCountDown <= yellowTime + redTime && cyclePhaseCountDown > redTime) {
            color = "yellow";
            if (turn == 20) {
                turnColor.put(turn, "red");
            }
        }
        if (cyclePhaseCountDown <= redTime) {
            color = "red";
            if (turn == 20) {
                turnColor.put(turn, "green");
            }
        }
        return color;
    }

    /**
     * 两秒倒计时，全场红
     *
     * @param turnColor
     * @param type
     * @param color
     */
    private static void setAllTurnColorRed(Map<Integer, String> turnColor, Integer type, String color) throws Exception {
        List<Integer> allTurn = Arrays.asList(1, 2, 3);
        for (Integer turnStr : allTurn) {
            if (!turnColor.containsKey(turnStr)) {
                turnColor.put(turnStr, "red");
            }
        }
        for (Map.Entry<Integer, String> entry : turnColor.entrySet()) {
            entry.setValue("red");
        }
    }

    /**
     * 圆饼灯设置所有方向灯色
     *
     * @param turnColor
     * @param color
     */
    private static void setAllTurnColor(Map<Integer, String> turnColor, String color) throws Exception {
        List<Integer> allTurn = Arrays.asList(1, 2);
        for (Integer turn : allTurn) {
            if (!turnColor.containsKey(turn)) {
                turnColor.put(turn, color);
            }
        }
    }

    /**
     * @param crossId
     * @param schemeNo
     * @param phaseNo
     * @return
     * @description 获取路口方案相位倒计时
     */
    private PhaseCountDownDTO getPhaseCountDownDTO(String crossId, String schemeNo, String phaseNo) {
        List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList = CrossSchemePhaseTimeCountCache.crossSchemePhaseCountDownList;
        if (!CollectionUtils.isEmpty(crossSchemePhaseCountDownList)) {
            for (CrossSchemePhaseCountDownDTO dto : crossSchemePhaseCountDownList) {
                if (Objects.equals(crossId, dto.getCrossId()) && Objects.equals(schemeNo, dto.getSchemeNo())) {
                    List<PhaseCountDownDTO> phaseCountDownDTOList = dto.getPhaseCountDownDTOList();
                    if (!CollectionUtils.isEmpty(phaseCountDownDTOList)) {
                        for (PhaseCountDownDTO phaseCountDownDTO : phaseCountDownDTOList) {
                            if (Objects.equals(phaseNo, phaseCountDownDTO.getPhaseNo())) {
                                return phaseCountDownDTO;
                            }
                        }
                    }
                }
            }
        }
        return null;
    }

    private static List<PhaseCountDownDTO> getPhaseCountDownList(String crossId, String schemeNo) {
        List<CrossSchemePhaseCountDownDTO> crossSchemePhaseCountDownList = CrossSchemePhaseTimeCountCache.crossSchemePhaseCountDownList;
        if (!CollectionUtils.isEmpty(crossSchemePhaseCountDownList)) {
            for (CrossSchemePhaseCountDownDTO dto : crossSchemePhaseCountDownList) {
                if (Objects.equals(crossId, dto.getCrossId()) && Objects.equals(schemeNo, dto.getSchemeNo())) {
                    List<PhaseCountDownDTO> phaseCountDownDTOList = dto.getPhaseCountDownDTOList();
                    if (!CollectionUtils.isEmpty(phaseCountDownDTOList)) {
                        return phaseCountDownDTOList;
                    }
                }
            }
        }
        return Collections.emptyList();
    }
}
